@@ -1,6 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
+
 import datetime
 from dateutil.relativedelta import relativedelta
 import itertools
@@ -12,7 +12,6 @@ from sql.conditionals import Coalesce
 from trytond.model import ModelView, Workflow, ModelSQL, fields, Check, Unique
 from trytond.wizard import Wizard, StateView, StateTransition, Button
 from trytond.pyson import Not, Equal, Eval, Or, Bool, If
-from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.tools import reduce_ids, grouped_slice
@@ -88,64 +87,31 @@ class Forecast(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'cancel': {
                     'invisible': Eval('state') != 'draft',
+                    'depends': ['state'],
                     },
                 'draft': {
                     'invisible': Eval('state') == 'draft',
+                    'depends': ['state'],
                     },
                 'confirm': {
                     'invisible': Eval('state') != 'draft',
+                    'depends': ['state'],
                     },
                 'complete': {
                     'readonly': Eval('state') != 'draft',
+                    'depends': ['state'],
                     },
                 })
+        cls._active_field = 'active'
 
     @classmethod
     def __register__(cls, module_name):
-        Location = Pool().get('stock.location')
-        TableHandler = backend.get('TableHandler')
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
-
-        table = TableHandler(cls, module_name)
-        migrate_warehouse = (not table.column_exist('warehouse')
-            and table.column_exist('location'))
-
         super(Forecast, cls).__register__(module_name)
 
         # Add index on create_date
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action('create_date', action='add')
 
-        if migrate_warehouse:
-            location2warehouse = {}
-
-            def find_warehouse(location):
-                if location.type == 'warehouse':
-                    return location.id
-                elif location.parent:
-                    return find_warehouse(location.parent)
-            cursor.execute(*sql_table.select(sql_table.id, sql_table.location))
-            for forecast_id, location_id in cursor.fetchall():
-                warehouse_id = location_id  # default fallback
-                if location_id in location2warehouse:
-                    warehouse_id = location2warehouse[location_id]
-                else:
-                    location = Location(location_id)
-                    warehouse_id = find_warehouse(location) or location_id
-                    location2warehouse[location_id] = warehouse_id
-                cursor.execute(*sql_table.update(
-                        columns=[sql_table.warehouse],
-                        values=[warehouse_id],
-                        where=sql_table.id == forecast_id))
-            table.not_null_action('warehouse',
-                action=cls.warehouse.required and 'add' or 'remove')
-            table.drop_column('location', True)
-
-        # Migration from 2.0 delete stock moves
-        forecasts = cls.search([])
-        cls.delete_moves(forecasts)
-
     @staticmethod
     def default_state():
         return 'draft'
@@ -229,13 +195,13 @@ class Forecast(Workflow, ModelSQL, ModelView):
                     })
 
     @classmethod
-    def delete(self, forecasts):
+    def delete(cls, forecasts):
         # Cancel before delete
-        self.cancel(forecasts)
+        cls.cancel(forecasts)
         for forecast in forecasts:
             if forecast.state != 'cancel':
-                self.raise_user_error('delete_cancel', forecast.rec_name)
-        super(Forecast, self).delete(forecasts)
+                cls.raise_user_error('delete_cancel', forecast.rec_name)
+        super(Forecast, cls).delete(forecasts)
 
     @classmethod
     @ModelView.button
@@ -279,26 +245,6 @@ class Forecast(Workflow, ModelSQL, ModelView):
         Line = Pool().get('stock.forecast.line')
         Line.delete_moves([l for f in forecasts for l in f.lines])
 
-    @classmethod
-    def copy(cls, forecasts, default=None):
-        Line = Pool().get('stock.forecast.line')
-
-        if default is None:
-            default = {}
-        default = default.copy()
-        default['lines'] = None
-
-        new_forecasts = []
-        for forecast in forecasts:
-            new_forecast, = super(Forecast, cls).copy([forecast],
-                default=default)
-            Line.copy([x for x in forecast.lines],
-                default={
-                    'forecast': new_forecast.id,
-                    })
-            new_forecasts.append(new_forecast)
-        return new_forecasts
-
 
 class ForecastLine(ModelSQL, ModelView):
     'Stock Forecast Line'
@@ -458,8 +404,9 @@ class ForecastLine(ModelSQL, ModelView):
     def copy(cls, lines, default=None):
         if default is None:
             default = {}
-        default = default.copy()
-        default['moves'] = None
+        else:
+            default = default.copy()
+        default.setdefault('moves', None)
         return super(ForecastLine, cls).copy(lines, default=default)
 
     def get_moves(self):
@@ -491,7 +438,7 @@ class ForecastLine(ModelSQL, ModelView):
                 unit_price, self.uom)
 
         moves = []
-        for day, qty in distribution.iteritems():
+        for day, qty in distribution.items():
             if qty == 0.0:
                 continue
             move = Move()
@@ -516,7 +463,7 @@ class ForecastLine(ModelSQL, ModelView):
 
     def distribute(self, delta, qty):
         'Distribute qty over delta'
-        range_delta = range(delta)
+        range_delta = list(range(delta))
         a = {}.fromkeys(range_delta, 0)
         while qty > 0:
             if qty > delta:
@@ -572,13 +519,13 @@ class ForecastComplete(Wizard):
     ask = StateView('stock.forecast.complete.ask',
         'stock_forecast.forecast_complete_ask_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Choose Products', 'choose', 'tryton-go-next'),
+            Button('Choose Products', 'choose', 'tryton-forward'),
             Button('Complete', 'complete', 'tryton-ok', default=True),
             ])
     choose = StateView('stock.forecast.complete.choose',
         'stock_forecast.forecast_complete_choose_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Choose Dates', 'ask', 'tryton-go-previous'),
+            Button('Choose Dates', 'ask', 'tryton-back'),
             Button('Complete', 'complete', 'tryton-ok', default=True),
             ])
     complete = StateTransition()
@@ -627,7 +574,7 @@ class ForecastComplete(Wizard):
             return {'products': [x.id for x in self.choose.products]}
         pbl = self._get_product_quantity()
         products = []
-        for (_, product), qty in pbl.iteritems():
+        for (_, product), qty in pbl.items():
             if qty < 0:
                 products.append(product)
         return {'products': products}
@@ -658,7 +605,7 @@ class ForecastComplete(Wizard):
             products = None
 
         to_save = []
-        for key, qty in pbl.iteritems():
+        for key, qty in pbl.items():
             _, product = key
             if products and product not in products:
                 continue
